@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: idlist.c,v 1.60.2.11 2005/03/31 06:07:10 priikone Exp $ */
+/* $Id: idlist.c,v 1.60.2.13 2005/04/11 21:26:30 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcclient.h"
@@ -700,6 +700,12 @@ SILC_CLIENT_CMD_FUNC(get_clients_by_channel_cb)
   SilcHashTableList htl;
   SilcChannelUser chu;
 
+  if (i->res_count) {
+    i->res_count--;
+    if (i->res_count)
+      return;
+  }
+
   channel = silc_client_get_channel_by_id(i->client, i->conn, &i->channel_id);
   if (channel && !silc_hash_table_count(channel->user_list)) {
     clients = silc_calloc(silc_hash_table_count(channel->user_list),
@@ -1132,8 +1138,8 @@ SilcChannelEntry silc_client_add_channel(SilcClient client,
 					     NULL, NULL, NULL, TRUE);
 
   /* Normalize channel name */
-  channel_namec = silc_identifier_check(channel_name, strlen(channel_name),
-					SILC_STRING_UTF8, 256, NULL);
+  channel_namec = silc_channel_name_check(channel_name, strlen(channel_name),
+					  SILC_STRING_UTF8, 256, NULL);
   if (!channel_namec) {
     silc_free(channel->channel_name);
     silc_hash_table_free(channel->user_list);
@@ -1240,9 +1246,9 @@ bool silc_client_replace_channel_id(SilcClient client,
   channel->id = new_id;
 
   /* Normalize channel name */
-  channel_namec = silc_identifier_check(channel->channel_name,
-					strlen(channel->channel_name),
-					SILC_STRING_UTF8, 256, NULL);
+  channel_namec = silc_channel_name_check(channel->channel_name,
+					  strlen(channel->channel_name),
+					  SILC_STRING_UTF8, 256, NULL);
   if (!channel_namec)
     return FALSE;
 
@@ -1269,8 +1275,8 @@ SilcChannelEntry silc_client_get_channel(SilcClient client,
   SILC_LOG_DEBUG(("Start"));
 
   /* Normalize name for search */
-  channel = silc_identifier_check(channel, strlen(channel), SILC_STRING_UTF8,
-				  256, NULL);
+  channel = silc_channel_name_check(channel, strlen(channel), SILC_STRING_UTF8,
+				    256, NULL);
   if (!channel)
     return NULL;
 
