@@ -1,10 +1,10 @@
 /*
 
-  silchashtable.c 
+  silchashtable.c
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 2001 - 2002 Pekka Riikonen
+  Copyright (C) 2001 - 2005 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -19,11 +19,11 @@
 /* Implementation of collision resistant hash table. This is a hash table
    that provides a reliable (what you add stays there, and duplicate keys
    are allowed) with as fast reference to the key as possible. If duplicate
-   keys are a lot in the hash table the lookup gets slower of course. 
+   keys are a lot in the hash table the lookup gets slower of course.
    However, this is reliable and no data is lost at any point. If you know
    that you never have duplicate keys then this is as fast as any simple
    hash table. */
-/* $Id: silchashtable.c,v 1.13 2002/02/01 20:48:07 priikone Exp $ */
+/* $Id: silchashtable.c,v 1.17.2.7 2005/05/02 08:50:09 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silchashtable.h"
@@ -38,7 +38,7 @@
 #endif
 
 /* Default size of the hash table (index to prime table) */
-#define SILC_HASH_TABLE_SIZE 3
+#define SILC_HASH_TABLE_SIZE 2
 
 /* Produce the index by hashing the key */
 #define SILC_HASH_TABLE_HASH(f, c) \
@@ -64,34 +64,35 @@ typedef struct SilcHashTableEntryStruct {
 /* Hash table. */
 struct SilcHashTableStruct {
   SilcHashTableEntry *table;
-  uint32 table_size;
-  uint32 entry_count;
+  SilcUInt32 table_size;
+  SilcUInt32 entry_count;
   SilcHashFunction hash;
   SilcHashCompare compare;
   SilcHashDestructor destructor;
   void *hash_user_context;
   void *compare_user_context;
   void *destructor_user_context;
-  bool auto_rehash;
+  unsigned int auto_rehash : 1;
 };
 
 /* Prime sizes for the hash table. The size of the table will always
    be one of these. */
-const uint32 primesize[42] = 
+const SilcUInt32 primesize[] =
 {
-  1, 3, 5, 11, 17, 37, 67, 109, 131, 163, 257, 367, 521, 823, 1031, 
-  1237, 2053, 2777, 4099, 6247, 8209, 14057, 16411, 21089, 32771, 47431,
-  65537, 106721, 131101, 262147, 360163, 524309, 810343, 1048583, 2097169,
-  4194319, 6153409, 8388617, 13845163, 16777259, 33554467, 67108879
+  3, 5, 11, 17, 37, 67, 109, 131, 163, 257, 367, 521, 823, 1031,
+  1237, 1447, 2053, 2389, 2777, 3323, 4099, 5059, 6247, 7001, 8209, 10993,
+  14057, 16411, 19181, 21089, 25033, 32771, 40009, 47431, 65537, 106721,
+  131101, 262147, 360163, 524309, 810343, 1048583, 2097169, 4194319,
+  6153409, 8388617, 13845163, 16777259, 33554467, 67108879
 };
 
 /* Find appropriate size for the hash table. The size will be a prime. */
 
-static uint32 silc_hash_table_primesize(uint32 size, uint32 *index)
+static SilcUInt32 silc_hash_table_primesize(SilcUInt32 size, SilcUInt32 *index)
 {
   int i;
 
-  for (i = 0; i < sizeof(primesize); i++)
+  for (i = 0; i < sizeof(primesize) / sizeof(primesize[0]); i++)
     if (primesize[i] >= size) {
       *index = i;
       SILC_HT_DEBUG(("sizeof of the hash table is %d", primesize[*index]));
@@ -110,11 +111,11 @@ static inline SilcHashTableEntry *
 silc_hash_table_find_internal(SilcHashTable ht, void *key,
 			      SilcHashTableEntry *prev_entry,
 			      SilcHashFunction hash, void *hash_user_context,
-			      SilcHashCompare compare, 
+			      SilcHashCompare compare,
 			      void *compare_user_context)
 {
   SilcHashTableEntry *entry, prev = NULL;
-  uint32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
+  SilcUInt32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
 
   SILC_HT_DEBUG(("index %d key %p", i, key));
 
@@ -136,19 +137,19 @@ silc_hash_table_find_internal(SilcHashTable ht, void *key,
 }
 
 /* Internal routine to find entry in the hash table by `key' and `context'.
-   Returns the previous entry (if exists) as well. */
+   Returns the previous entry (if exists) as well to `prev_entry'. */
 
 static inline SilcHashTableEntry *
 silc_hash_table_find_internal_context(SilcHashTable ht, void *key,
 				      void *context,
 				      SilcHashTableEntry *prev_entry,
-				      SilcHashFunction hash, 
+				      SilcHashFunction hash,
 				      void *hash_user_context,
-				      SilcHashCompare compare, 
+				      SilcHashCompare compare,
 				      void *compare_user_context)
 {
   SilcHashTableEntry *entry, prev = NULL;
-  uint32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
+  SilcUInt32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
 
   SILC_HT_DEBUG(("index %d key %p context %p", i, key, context));
 
@@ -170,7 +171,8 @@ silc_hash_table_find_internal_context(SilcHashTable ht, void *key,
     }
   }
 
-  *prev_entry = prev;
+  if (prev_entry)
+    *prev_entry = prev;
   return entry;
 }
 
@@ -184,7 +186,7 @@ silc_hash_table_find_internal_simple(SilcHashTable ht, void *key,
 				     void *compare_user_context)
 {
   SilcHashTableEntry *entry;
-  uint32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
+  SilcUInt32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
 
   SILC_HT_DEBUG(("index %d key %p", i, key));
 
@@ -205,7 +207,7 @@ silc_hash_table_find_internal_simple(SilcHashTable ht, void *key,
    hash and comparison functions. */
 
 static inline void
-silc_hash_table_find_internal_all(SilcHashTable ht, void *key, 
+silc_hash_table_find_internal_all(SilcHashTable ht, void *key,
 				  SilcHashFunction hash,
 				  void *hash_user_context,
 				  SilcHashCompare compare,
@@ -213,9 +215,9 @@ silc_hash_table_find_internal_all(SilcHashTable ht, void *key,
 				  SilcHashForeach foreach,
 				  void *foreach_user_context)
 {
-  SilcHashTableEntry *entry, *tmp;
-  bool auto_rehash;
-  uint32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
+  SilcHashTableEntry e, tmp;
+  bool auto_rehash, found = FALSE;
+  SilcUInt32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
 
   SILC_HT_DEBUG(("index %d key %p", i, key));
 
@@ -224,41 +226,43 @@ silc_hash_table_find_internal_all(SilcHashTable ht, void *key,
   auto_rehash = ht->auto_rehash;
   ht->auto_rehash = FALSE;
 
-  entry = &ht->table[i];
+  e = ht->table[i];
   if (compare) {
-    while (*entry) {
-      if (compare((*entry)->key, key, compare_user_context)) {
-	tmp = &(*entry)->next;
-	foreach((*entry)->key, (*entry)->context, foreach_user_context);
-	entry = tmp;
-	continue;
+    while (e) {
+      tmp = e->next;
+      if (compare(e->key, key, compare_user_context)) {
+	found = TRUE;
+	foreach(e->key, e->context, foreach_user_context);
       }
-      entry = &(*entry)->next;
+      e = tmp;
     }
   } else {
-    while (*entry) {
-      if ((*entry)->key == key) {
-	tmp = &(*entry)->next;
-	foreach((*entry)->key, (*entry)->context, foreach_user_context);
-	entry = tmp;
-	continue;
+    while (e) {
+      tmp = e->next;
+      if (e->key == key) {
+	found = TRUE;
+	foreach(e->key, e->context, foreach_user_context);
       }
-      entry = &(*entry)->next;
+      e = tmp;
     }
   }
 
+  /* If nothing was found call with NULL context the callback */
+  if (!found)
+    foreach(key, NULL, foreach_user_context);
+
   ht->auto_rehash = auto_rehash;
 }
 
 /* Internal routine to add new key to the hash table */
 
-static inline void
+static inline bool
 silc_hash_table_add_internal(SilcHashTable ht, void *key, void *context,
-			     SilcHashFunction hash, 
+			     SilcHashFunction hash,
 			     void *hash_user_context)
 {
   SilcHashTableEntry *entry;
-  uint32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
+  SilcUInt32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
 
   SILC_HT_DEBUG(("index %d key %p", i, key));
 
@@ -278,6 +282,8 @@ silc_hash_table_add_internal(SilcHashTable ht, void *key, void *context,
     SILC_HT_DEBUG(("Collision; adding new key to list"));
 
     e->next = silc_calloc(1, sizeof(*e->next));
+    if (!e->next)
+      return FALSE;
     e->next->key = key;
     e->next->context = context;
     ht->entry_count++;
@@ -285,6 +291,8 @@ silc_hash_table_add_internal(SilcHashTable ht, void *key, void *context,
     /* New key */
     SILC_HT_DEBUG(("New key"));
     *entry = silc_calloc(1, sizeof(**entry));
+    if (!(*entry))
+      return FALSE;
     (*entry)->key = key;
     (*entry)->context = context;
     ht->entry_count++;
@@ -292,17 +300,19 @@ silc_hash_table_add_internal(SilcHashTable ht, void *key, void *context,
 
   if (SILC_HASH_REHASH_INC)
     silc_hash_table_rehash(ht, 0);
+
+  return TRUE;
 }
 
 /* Internal routine to replace old key with new one (if it exists) */
 
-static inline void
+static inline bool
 silc_hash_table_replace_internal(SilcHashTable ht, void *key, void *context,
-				 SilcHashFunction hash, 
+				 SilcHashFunction hash,
 				 void *hash_user_context)
 {
   SilcHashTableEntry *entry;
-  uint32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
+  SilcUInt32 i = SILC_HASH_TABLE_HASH(hash, hash_user_context);
 
   SILC_HT_DEBUG(("index %d key %p", i, key));
 
@@ -311,11 +321,13 @@ silc_hash_table_replace_internal(SilcHashTable ht, void *key, void *context,
     /* The entry exists already. We have a collision, replace the old
        key and context. */
     if (ht->destructor)
-      ht->destructor((*entry)->key, (*entry)->context, 
+      ht->destructor((*entry)->key, (*entry)->context,
 		     ht->destructor_user_context);
   } else {
     /* New key */
     *entry = silc_calloc(1, sizeof(**entry));
+    if (!(*entry))
+      return FALSE;
     ht->entry_count++;
   }
 
@@ -324,6 +336,8 @@ silc_hash_table_replace_internal(SilcHashTable ht, void *key, void *context,
 
   if (SILC_HASH_REHASH_INC)
     silc_hash_table_rehash(ht, 0);
+
+  return TRUE;
 }
 
 /* Allocates new hash table and returns it.  If the `table_size' is not
@@ -334,7 +348,7 @@ silc_hash_table_replace_internal(SilcHashTable ht, void *key, void *context,
    destructor function, respectively. The `hash' is mandatory, the others
    are optional. */
 
-SilcHashTable silc_hash_table_alloc(uint32 table_size, 
+SilcHashTable silc_hash_table_alloc(SilcUInt32 table_size,
 				    SilcHashFunction hash,
 				    void *hash_user_context,
 				    SilcHashCompare compare,
@@ -344,16 +358,22 @@ SilcHashTable silc_hash_table_alloc(uint32 table_size,
 				    bool auto_rehash)
 {
   SilcHashTable ht;
-  uint32 size_index = SILC_HASH_TABLE_SIZE;
+  SilcUInt32 size_index = SILC_HASH_TABLE_SIZE;
 
   if (!hash)
     return NULL;
 
   ht = silc_calloc(1, sizeof(*ht));
+  if (!ht)
+    return NULL;
   ht->table = silc_calloc(table_size ? silc_hash_table_primesize(table_size,
 								 &size_index) :
 			  primesize[SILC_HASH_TABLE_SIZE],
 			  sizeof(*ht->table));
+  if (!ht->table) {
+    silc_free(ht);
+    return NULL;
+  }
   ht->table_size = size_index;
   ht->hash = hash;
   ht->compare = compare;
@@ -391,7 +411,7 @@ void silc_hash_table_free(SilcHashTable ht)
 
 /* Returns the size of the hash table */
 
-uint32 silc_hash_table_size(SilcHashTable ht)
+SilcUInt32 silc_hash_table_size(SilcHashTable ht)
 {
   return primesize[ht->table_size];
 }
@@ -400,7 +420,7 @@ uint32 silc_hash_table_size(SilcHashTable ht)
    entries in the table thatn the size of the hash table calling the
    silc_hash_table_rehash is recommended. */
 
-uint32 silc_hash_table_count(SilcHashTable ht)
+SilcUInt32 silc_hash_table_count(SilcHashTable ht)
 {
   return ht->entry_count;
 }
@@ -412,7 +432,7 @@ uint32 silc_hash_table_count(SilcHashTable ht)
 
 void silc_hash_table_add(SilcHashTable ht, void *key, void *context)
 {
-  silc_hash_table_add_internal(ht, key, context, ht->hash, 
+  silc_hash_table_add_internal(ht, key, context, ht->hash,
 			       ht->hash_user_context);
 }
 
@@ -431,14 +451,14 @@ void silc_hash_table_add_ext(SilcHashTable ht, void *key, void *context,
 
 void silc_hash_table_replace(SilcHashTable ht, void *key, void *context)
 {
-  silc_hash_table_replace_internal(ht, key, context, ht->hash, 
+  silc_hash_table_replace_internal(ht, key, context, ht->hash,
 				   ht->hash_user_context);
 }
 
 /* Same as above but with specific hash function. */
 
 void silc_hash_table_replace_ext(SilcHashTable ht, void *key, void *context,
-				 SilcHashFunction hash, 
+				 SilcHashFunction hash,
 				 void *hash_user_context)
 {
   silc_hash_table_replace_internal(ht, key, context, hash, hash_user_context);
@@ -484,9 +504,9 @@ bool silc_hash_table_del(SilcHashTable ht, void *key)
 /* Same as above but with specific hash and compare functions. */
 
 bool silc_hash_table_del_ext(SilcHashTable ht, void *key,
-			     SilcHashFunction hash, 
+			     SilcHashFunction hash,
 			     void *hash_user_context,
-			     SilcHashCompare compare, 
+			     SilcHashCompare compare,
 			     void *compare_user_context,
 			     SilcHashDestructor destructor,
 			     void *destructor_user_context)
@@ -498,7 +518,7 @@ bool silc_hash_table_del_ext(SilcHashTable ht, void *key,
 					hash_user_context ? hash_user_context :
 					ht->hash_user_context,
 					compare ? compare : ht->compare,
-					compare_user_context ? 
+					compare_user_context ?
 					compare_user_context :
 					ht->compare_user_context);
   if (*entry == NULL)
@@ -536,13 +556,13 @@ bool silc_hash_table_del_ext(SilcHashTable ht, void *key,
    have duplicate keys. In that case the `context' may be used to check
    whether the correct entry is being deleted. */
 
-bool silc_hash_table_del_by_context(SilcHashTable ht, void *key, 
+bool silc_hash_table_del_by_context(SilcHashTable ht, void *key,
 				    void *context)
 {
   SilcHashTableEntry *entry, prev, e;
 
   entry = silc_hash_table_find_internal_context(ht, key, context, &prev,
-						ht->hash, 
+						ht->hash,
 						ht->hash_user_context,
 						ht->compare,
 						ht->compare_user_context);
@@ -574,11 +594,11 @@ bool silc_hash_table_del_by_context(SilcHashTable ht, void *key,
 
 /* Same as above but with specific hash and compare functions. */
 
-bool silc_hash_table_del_by_context_ext(SilcHashTable ht, void *key, 
+bool silc_hash_table_del_by_context_ext(SilcHashTable ht, void *key,
 					void *context,
-					SilcHashFunction hash, 
+					SilcHashFunction hash,
 					void *hash_user_context,
-					SilcHashCompare compare, 
+					SilcHashCompare compare,
 					void *compare_user_context,
 					SilcHashDestructor destructor,
 					void *destructor_user_context)
@@ -587,12 +607,12 @@ bool silc_hash_table_del_by_context_ext(SilcHashTable ht, void *key,
 
   entry = silc_hash_table_find_internal_context(ht, key, context, &prev,
 						hash ? hash : ht->hash,
-						hash_user_context ? 
+						hash_user_context ?
 						hash_user_context :
 						ht->hash_user_context,
-						compare ? 
+						compare ?
 						compare : ht->compare,
-						compare_user_context ? 
+						compare_user_context ?
 						compare_user_context :
 						ht->compare_user_context);
   if (*entry == NULL)
@@ -626,19 +646,38 @@ bool silc_hash_table_del_by_context_ext(SilcHashTable ht, void *key,
 }
 
 /* Finds the entry in the hash table by the provided `key' as fast as
-   possible. Return TRUE if the entry was found and FALSE otherwise. 
+   possible. Return TRUE if the entry was found and FALSE otherwise.
    The found entry is returned to the `ret_key' and `ret_context',
    respectively. If the `ret_key and `ret_context' are NULL then this
    maybe used only to check whether given key exists in the table. */
 
 bool silc_hash_table_find(SilcHashTable ht, void *key,
 			  void **ret_key, void **ret_context)
+{
+  return silc_hash_table_find_ext(ht, key, ret_key, ret_context,
+				  NULL, NULL, NULL, NULL);
+}
+
+/* Same as above but with specified hash and comparison functions. */
+
+bool silc_hash_table_find_ext(SilcHashTable ht, void *key,
+			      void **ret_key, void **ret_context,
+			      SilcHashFunction hash,
+			      void *hash_user_context,
+			      SilcHashCompare compare,
+			      void *compare_user_context)
 {
   SilcHashTableEntry *entry;
 
-  entry = silc_hash_table_find_internal_simple(ht, key, ht->hash, 
+  entry = silc_hash_table_find_internal_simple(ht, key,
+					       hash ? hash : ht->hash,
+					       hash_user_context ?
+					       hash_user_context :
 					       ht->hash_user_context,
-					       ht->compare, 
+					       compare ? compare :
+					       ht->compare,
+					       compare_user_context ?
+					       compare_user_context :
 					       ht->compare_user_context);
   if (*entry == NULL)
     return FALSE;
@@ -651,34 +690,41 @@ bool silc_hash_table_find(SilcHashTable ht, void *key,
   return TRUE;
 }
 
+/* Same as silc_hash_table_find but finds with specific context. */
+
+bool silc_hash_table_find_by_context(SilcHashTable ht, void *key,
+				     void *context, void **ret_key)
+{
+  return silc_hash_table_find_by_context_ext(ht, key, context, ret_key,
+					     NULL, NULL, NULL, NULL);
+}
+
 /* Same as above but with specified hash and comparison functions. */
 
-bool silc_hash_table_find_ext(SilcHashTable ht, void *key,
-			      void **ret_key, void **ret_context,
-			      SilcHashFunction hash, 
-			      void *hash_user_context,
-			      SilcHashCompare compare, 
-			      void *compare_user_context)
+bool silc_hash_table_find_by_context_ext(SilcHashTable ht, void *key,
+					 void *context, void **ret_key,
+					 SilcHashFunction hash,
+					 void *hash_user_context,
+					 SilcHashCompare compare,
+					 void *compare_user_context)
 {
   SilcHashTableEntry *entry;
 
-  entry = silc_hash_table_find_internal_simple(ht, key,
-					       hash ? hash : ht->hash, 
-					       hash_user_context ? 
-					       hash_user_context :
-					       ht->hash_user_context,
-					       compare ? compare :
-					       ht->compare, 
-					       compare_user_context ?
-					       compare_user_context :
-					       ht->compare_user_context);
-  if (*entry == NULL)
+  entry = silc_hash_table_find_internal_context(ht, key, context, NULL,
+						hash ? hash : ht->hash,
+						hash_user_context ?
+						hash_user_context :
+						ht->hash_user_context,
+						compare ? compare :
+						ht->compare,
+						compare_user_context ?
+						compare_user_context :
+						ht->compare_user_context);
+  if (!entry || !(*entry))
     return FALSE;
 
   if (ret_key)
     *ret_key = (*entry)->key;
-  if (ret_context)
-    *ret_context = (*entry)->context;
 
   return TRUE;
 }
@@ -699,20 +745,20 @@ void silc_hash_table_find_foreach(SilcHashTable ht, void *key,
 /* Same as above but with specific hash and comparison functions. */
 
 void silc_hash_table_find_foreach_ext(SilcHashTable ht, void *key,
-				      SilcHashFunction hash, 
+				      SilcHashFunction hash,
 				      void *hash_user_context,
-				      SilcHashCompare compare, 
+				      SilcHashCompare compare,
 				      void *compare_user_context,
-				      SilcHashForeach foreach, 
+				      SilcHashForeach foreach,
 				      void *foreach_user_context)
 {
   silc_hash_table_find_internal_all(ht, key,
-				    hash ? hash : ht->hash, 
-				    hash_user_context ? 
+				    hash ? hash : ht->hash,
+				    hash_user_context ?
 				    hash_user_context :
 				    ht->hash_user_context,
 				    compare ? compare :
-				    ht->compare, 
+				    ht->compare,
 				    compare_user_context ?
 				    compare_user_context :
 				    ht->compare_user_context,
@@ -751,11 +797,12 @@ void silc_hash_table_foreach(SilcHashTable ht, SilcHashForeach foreach,
    the new table of a suitable size. Note that this operation may be
    very slow. */
 
-void silc_hash_table_rehash(SilcHashTable ht, uint32 new_size)
+void silc_hash_table_rehash(SilcHashTable ht, SilcUInt32 new_size)
 {
   int i;
   SilcHashTableEntry *table, e, tmp;
-  uint32 table_size, size_index;
+  SilcUInt32 table_size, size_index;
+  bool auto_rehash;
 
   SILC_HT_DEBUG(("Start"));
 
@@ -772,9 +819,13 @@ void silc_hash_table_rehash(SilcHashTable ht, uint32 new_size)
   /* Take old hash table */
   table = ht->table;
   table_size = ht->table_size;
+  auto_rehash = ht->auto_rehash;
+  ht->auto_rehash = FALSE;
 
   /* Allocate new table */
   ht->table = silc_calloc(primesize[size_index], sizeof(*ht->table));
+  if (!ht->table)
+    return;
   ht->table_size = size_index;
   ht->entry_count = 0;
 
@@ -791,19 +842,22 @@ void silc_hash_table_rehash(SilcHashTable ht, uint32 new_size)
     }
   }
 
+  ht->auto_rehash = auto_rehash;
+
   /* Remove old table */
   silc_free(table);
 }
 
 /* Same as above but with specific hash function. */
 
-void silc_hash_table_rehash_ext(SilcHashTable ht, uint32 new_size,
-				SilcHashFunction hash, 
+void silc_hash_table_rehash_ext(SilcHashTable ht, SilcUInt32 new_size,
+				SilcHashFunction hash,
 				void *hash_user_context)
 {
   int i;
   SilcHashTableEntry *table, e, tmp;
-  uint32 table_size, size_index;
+  SilcUInt32 table_size, size_index;
+  bool auto_rehash;
 
   SILC_HT_DEBUG(("Start"));
 
@@ -820,9 +874,13 @@ void silc_hash_table_rehash_ext(SilcHashTable ht, uint32 new_size,
   /* Take old hash table */
   table = ht->table;
   table_size = ht->table_size;
+  auto_rehash = ht->auto_rehash;
+  ht->auto_rehash = FALSE;
 
   /* Allocate new table */
   ht->table = silc_calloc(primesize[size_index], sizeof(*ht->table));
+  if (!ht->table)
+    return;
   ht->table_size = size_index;
   ht->entry_count = 0;
 
@@ -830,7 +888,7 @@ void silc_hash_table_rehash_ext(SilcHashTable ht, uint32 new_size,
   for (i = 0; i < primesize[table_size]; i++) {
     e = table[i];
     while (e) {
-      silc_hash_table_add_ext(ht, e->key, e->context, hash, 
+      silc_hash_table_add_ext(ht, e->key, e->context, hash,
 			      hash_user_context);
       tmp = e;
       e = e->next;
@@ -840,6 +898,8 @@ void silc_hash_table_rehash_ext(SilcHashTable ht, uint32 new_size,
     }
   }
 
+  ht->auto_rehash = auto_rehash;
+
   /* Remove old table */
   silc_free(table);
 }
